@@ -574,18 +574,31 @@ def test_sdpa(
     # batch size
     b = 2
     # query sequence length
-    s_q = random.choice([8, 16, 24, 32, 256, 512, 1024, 2048])
+    if cudnn_version >= "9.7.0":
+        s_q = random.choice([24, 256, 512, 1024, 2048]) # Need to add s_q = 1 to the harness
+    else:
+        s_q = random.choice([24, 256, 512, 1024, 2048])
+
+    if s_q == 1:
+        is_left_bound = False
+        request.config.option.mha_left_bound = None
+        
     # key+value sequence length
     s_kv = (
-        random.choice([8, 16, 24, 32, 256, 512, 1024, 2048])
-        if layout == "bshd_bshd_bshd"
+        random.choice([24, 32, 256, 512, 1024, 2048])
+        if layout == "bshd_bshd_bshd" or s_q == 1
         else s_q
     )
+
+    d_choices = [32, 56, 64, 128]
+    # if cudnn_version >= "9.9.0":
+    #     d_choices.extend([192, 256, 512])
+
     # query+key embedding dimension per head
-    d_qk = random.choice([32, 56, 64, 128])
+    d_qk = random.choice(d_choices)
     # value embedding dimension per head
     d_v = (
-        random.choice([64, 96, 128])
+        random.choice(d_choices)
         if (layout == "bshd_bshd_bshd" and not is_ragged)
         else d_qk
     )
@@ -610,8 +623,8 @@ def test_sdpa(
     assert (request.config.option.mha_left_bound is None) or is_left_bound
     assert (request.config.option.mha_right_bound is None) or is_right_bound
 
-    # If bounds are requested: randomly pick between 0 and s_kv/4
-    left_bound = random.choice([1, s_kv//4]) if is_left_bound else None
+    # If bounds are requested: randomly pick between 1/0 and s_kv/4
+    left_bound = max(1, random.choice([1, s_kv//4])) if is_left_bound else None
     right_bound = random.choice([0, s_kv//4]) if is_right_bound else None
 
     # -------------------------- override test parameters if args are provided ----------------
@@ -627,6 +640,10 @@ def test_sdpa(
     left_bound = int(request.config.option.mha_left_bound) if request.config.option.mha_left_bound != None else left_bound
     right_bound = int(request.config.option.mha_right_bound) if request.config.option.mha_right_bound != None else right_bound
 
+    if s_q == 1:
+        is_dropout = False
+        request.config.option.mha_dropout = None
+
     if d_qk != d_v and cudnn_version < "8.9.6":
         pytest.skip("d_qk != d_v is only supported on 8.9.6 onwards.")
 
@@ -732,8 +749,15 @@ def test_sdpa(
         reshaped = torch.cat((cat_tensor.clone()).chunk(blocks_per_batch, dim=2), dim=0)
 
         table_size = math.ceil(S/block_size)
-        page_table = torch.linspace(0, B*table_size-1, B*table_size, device='cuda', dtype=torch.int32).reshape(table_size,1,B,1)
-        page_table = torch.transpose(page_table,0,2)
+        page_table_temp = torch.linspace(0, B*table_size-1, B*table_size, device='cuda', dtype=torch.int32).reshape(table_size,1,B,1)
+        page_table_temp = torch.transpose(page_table_temp,0,2)
+
+        alt_stride = (blocks_per_batch, blocks_per_batch, 1, 1)
+
+        # Make batch size outer dimension (required for SM100 kernel)
+        page_table_dims = (B, 1, blocks_per_batch, 1)
+        page_table = torch.randn(blocks_per_batch * B).int().cuda().as_strided(page_table_dims, alt_stride)
+        page_table.copy_(page_table_temp)
 
         return(reshaped, page_table)
 
@@ -1031,8 +1055,8 @@ def test_sdpa_backward(
     assert (request.config.option.mha_left_bound is None) or is_left_bound
     assert (request.config.option.mha_right_bound is None) or is_right_bound
 
-    # If bounds are requested: randomly pick between 0 and s_kv/4
-    left_bound = random.choice([1, s_kv//4]) if is_left_bound else None
+    # If bounds are requested: randomly pick between 1/0 and s_kv/4
+    left_bound = max(1, random.choice([1, s_kv//4])) if is_left_bound else None
     right_bound = random.choice([0, s_kv//4]) if is_right_bound else None
 
     # -------------------------- override test parameters if args are provided ----------------
